package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.entity.PmProject;
import com.nationrel.modules.project.entity.PmTask;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.config.SysConfig;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.PmMapKeySet;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.exception.ApplicationRuntimeException;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.mapper.PmProjectMapper;
import com.nationrel.modules.project.mapper.PmTaskMapper;
import com.nationrel.modules.project.mapper.PmTgMapper;
import com.nationrel.modules.project.service.PmPercentStatService;
import com.nationrel.modules.project.service.PmStageCtrlService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName PmStageCtrlServiceImpl
 * @Description 可靠性项目阶段服务实现类
 * @Author ted.liu
 * @CreateDate 2013-12-10
 */
@Service
public class PmStageCtrlServiceImpl implements PmStageCtrlService {

    @Autowired
    private SysConfig sysConfig;
    @Autowired
    private PmProjectMapper pmProjectMapper;
    @Autowired
    private PmTgMapper pmTgMapper;
    @Autowired
    private PmTaskMapper pmTaskMapper;
    @Autowired
    private PmPercentStatService pmPercentStatService;

    /**
     * @param projectId:项目标识
     * @return void:
     * E300101 代表:由于TG没有完成，项目不能向完成状态迁移;
     * E300111 代表:由于项目下没有TG，当前项目不能向完成状态迁移。
     * 其他情况返回null.
     * @Title: ctrlProjectState
     * @Description: 控制项目的状态是否可推进
     */
    @Override
    public void ctrlProjectState(String projectId) throws Exception {

        // 1.获取项目信息
        PmProject pmProject = pmProjectMapper.selectByPrimaryKey(projectId);

        // 2.如果当前项目的状态不等于工作中，则其状态迁移无必要条件
        if (!ReliaFlowConstants.LIFECYCLE_RELIA_PJ_INWORK.equals(pmProject.getLifeState())) {
        } else {

            // 3.项目的状态等于工作中
            List<PmTg> pmTgList = pmTgMapper.selectPmTgByProjectId(projectId);
            if (pmTgList == null || pmTgList.size() <= 0) {
                throw new ApplicationRuntimeException(MessageId.E300111);
            } else {
                // 判断其下所有TG是否全部完成.
                for (PmTg pmTg : pmTgList) {
                    if (!ReliaFlowConstants.LIFECYCLE_RELIA_TG_COMPLETE.equals(pmTg.getLifeState())) {
                        throw new ApplicationRuntimeException(MessageId.E300101);
                    }
                }
            }

        }
    }

    /**
     * @param projectId 项目标识
     * @param tgId      TG标识
     * @return void:
     * E300102代表:由于任务没有完成，TG不能向完成状态迁移
     * E300103代表:由于TG下不存在任务，TG不能向完成状态迁移
     * E300104代表:由于项目没有进入工作状态，TG不能向工作状态迁移;
     * E300105代表:由于前面TG没有完成，当前TG不能向工作状态迁移;
     * 其他情况返回null.
     * @Title: ctrlTgState
     * @Description: 控制TG的状态是否可推进
     */
    @Override
    public void ctrlTgState(String projectId, String tgId) throws Exception {

        // 1.获取项目信息
        PmProject pmProject = pmProjectMapper.selectByPrimaryKey(projectId);

        // 2.如果当前项目的状态不在工作中,则当前TG不可推进
        if (ReliaFlowConstants.LIFECYCLE_RELIA_PJ_INWORK.compareTo(pmProject.getLifeState()) > 0) {
            throw new ApplicationRuntimeException(MessageId.E300104);
        }

        // 3.获取TG信息
        PmTg pmTg = pmTgMapper.selectByPrimaryKey(tgId);

        // 4.如果当前TG的状态不在工作中
        if (!ReliaFlowConstants.LIFECYCLE_RELIA_TG_INWORK.equals(pmTg.getLifeState())) {

            // 如果当前TG的上一个TG尚未完成,则当前TG的状态不能进入工作中
            PmTg prevPmTg = null;
            List<PmTg> pmTgs = pmTgMapper.selectPmTgByProjectId(pmTg.getProjectId());
            for (int i = 0; i < pmTgs.size(); i++) {
                if (pmTgs.get(i).getTgId().equals(tgId) && i != 0) {
                    prevPmTg = pmTgs.get(--i);
                    break;
                }
            }

            if (prevPmTg != null && !ReliaFlowConstants.LIFECYCLE_RELIA_TG_COMPLETE.equals(prevPmTg.getLifeState())) {
                throw new ApplicationRuntimeException(MessageId.E300105);
            }
        }

        // 5.如果TG的状态在工作中,则判断其下所有任务是否都已完成.
        if (ReliaFlowConstants.LIFECYCLE_RELIA_TG_INWORK.equals(pmTg.getLifeState())) {

            List<PmTask> taskList = pmTaskMapper.selectPmTaskByTgId(tgId);
            if (taskList == null || taskList.size() <= Constants.C_NUM_0) {
                /*
                 * 目前允许即使TG下没有任务也可从工作中向完成推移(2014-03-25注释掉以下代码)
                 * retMsg = MessageId.E300103;
                   return retMsg;
                 */
                throw new ApplicationRuntimeException(MessageId.E300103);
            } else {
                for (PmTask task : taskList) {
                    if (!ReliaFlowConstants.LIFECYCLE_RELIA_TASK_COMPLETE.equals(task.getLifeState())) {
                        throw new ApplicationRuntimeException(MessageId.E300102);
                    }
                }
            }
        }
    }

    /**
     * @param tgId:   TG标识
     * @param taskId: Task标识
     * @return void:
     * E300109  代表:由于父任务没有完成，当前任务不能向工作状态迁移;
     * E300108  代表:由于TG没有进入工作状态，任务不能向工作状态迁移;
     * E300110  代表:由于子任务没有全部完成，当前任务不能向完成状态迁移;
     * E300112  代表:任务完成度必须大于等于系统允许的最小进度;
     * 其他情况,返回null,即允许Task的状态推进.
     * @Title: ctrlTaskState
     * @Description: 控制Task的状态是否可推进
     */
    @Override
    public void ctrlTaskState(String tgId, String taskId) throws Exception {

        // 获取任务信息
        Map<String, Object> task = pmTaskMapper.getTaskWholeInfoByTaskId(taskId);
        if (task == null || task.size() <= Constants.C_NUM_0) {
            throw new OptimisticLockException(MessageId.W300104);
        }

        // 当前任务的状态为创建
        if (task.get(PmMapKeySet.KEY_R_LIFE_STATE) != null && task.get(PmMapKeySet.KEY_R_LIFE_STATE).toString().equals(ReliaFlowConstants.LIFECYCLE_RELIA_TASK_CREATE)) {
            if (task.get(PmMapKeySet.KEY_PARENT_TASK_ID) != null && task.get(PmMapKeySet.KEY_PARENT_TASK_ID).equals(ReliaFlowConstants.TASK_FIRST_PARENT_ID)) {

                // 当前任务为一级任务,则判断其所属TG的状态是否为工作中
                PmTg taskOfTg = pmTgMapper.selectByPrimaryKey(task.get(PmMapKeySet.KEY_TG_ID).toString());
                if (taskOfTg != null && !taskOfTg.getLifeState().equals(ReliaFlowConstants.LIFECYCLE_RELIA_TG_INWORK)) {
                    throw new ApplicationRuntimeException(MessageId.E300108);
                }
            } else if (task.get(PmMapKeySet.KEY_PARENT_TASK_ID) != null && !task.get(PmMapKeySet.KEY_PARENT_TASK_ID).equals(ReliaFlowConstants.TASK_FIRST_PARENT_ID)) {

                // 当前任务非一级任务,则判断其直属父任务的状态是否为工作中
                String directTaskId = task.get(PmMapKeySet.KEY_PARENT_TASK_ID).toString();
                Map<String, Object> directTask = pmTaskMapper.getTaskWholeInfoByTaskId(directTaskId);
                if (directTask == null || directTask.get(PmMapKeySet.KEY_R_LIFE_STATE) == null) {
                    throw new OptimisticLockException(MessageId.W009803);
                }
                if (!directTask.get(PmMapKeySet.KEY_R_LIFE_STATE).toString().equals(ReliaFlowConstants.LIFECYCLE_RELIA_TASK_INWORK)) {
                    throw new ApplicationRuntimeException(MessageId.E300109);
                }

            }
        } else if (task.get(PmMapKeySet.KEY_R_LIFE_STATE) != null && task.get(PmMapKeySet.KEY_R_LIFE_STATE).toString().equals(ReliaFlowConstants.LIFECYCLE_RELIA_TASK_INWORK)) {

            /*
             * 如果当前任务的状态为工作中,需进行如下处理
             * 1.当前任务进度(任务进度来源可以是页面输入的或调用统计服务得到的。先判断是不是页面输入)必须达到一定值，
             *   才可以结束.--2014-03-25添加当前条件
             * 2.判断其下所有子任务是否全部 完成
             */
            BigDecimal percent = null;
            Object percentFromDB = task.get(PmMapKeySet.KEY_R_PERCENT);
            if (percentFromDB == null || new BigDecimal(percentFromDB.toString()).compareTo(new BigDecimal(Constants.C_NUM_0)) <= 0) {

                // 如果任务的进度为null或者零，则调用服务，计算任务进度
                Map<String, Object> argPercentMap = new HashMap<String, Object>();
                argPercentMap.put(PmMapKeySet.KEY_S_TG_ID, task.get(PmMapKeySet.KEY_TG_ID));
                argPercentMap.put(PmMapKeySet.KEY_S_START_NODE_ID, task.get(PmMapKeySet.KEY_R_TASK_ID));
                percent = pmPercentStatService.statCurrentNodePercent(argPercentMap);
            } else {
                percent = new BigDecimal(percentFromDB.toString());
            }
            BigDecimal minPercent = new BigDecimal(sysConfig.getMinPercent());
            if (percent.compareTo(minPercent) < 0) {

                // 任务完成度必须大于等于系统允许的最小进度
                throw new ApplicationRuntimeException(MessageId.E300112);
            } else {

                // 其下所有子任务是否全部完成
                if (judgeChildTaskStage(task.get(PmMapKeySet.KEY_R_TASK_ID).toString())) {
                    throw new ApplicationRuntimeException(MessageId.E300110);
                }
            }

        }
    }

    /**
     * @param task
     * @return boolean: false:当前任务下存在没有完成的子任务;
     * true:当前任务下所有子任务已全部完成.
     * @Title: judgeChildTaskStage
     * @Description: 递归当前任务下所有子任务, 判断其下子任务的状态是否全部完成.
     */
    private boolean judgeChildTaskStage(String taskId) {
        boolean result = false;

        // 获取当前任务下直接子任务集合
        List<Map<String, Object>> directChildeTaskList = pmTaskMapper.getDirectChildTaskList(taskId);
        if (directChildeTaskList == null || directChildeTaskList.size() <= 0) {

        } else {
            for (Map<String, Object> childTask : directChildeTaskList) {
                if (!childTask.get(PmMapKeySet.KEY_R_LIFE_STATE).toString().equals(ReliaFlowConstants.LIFECYCLE_RELIA_TASK_COMPLETE)) {
                    result = true;
                    break;
                }
            }
        }
        return result;
    }

    /**
     * @param taskId
     * @return boolean
     * @Title: hasChildTask
     * @Description: 判断给定的Task下是否有子任务
     */
    private boolean hasChildTask(String taskId) {
        int directChildTaskNum = pmTaskMapper.countDirectChildTaskNumber(taskId);
        return directChildTaskNum > 0;
    }
}
