package com.nationrel.modules.project.service.impl;

import com.nationrel.modules.project.entity.PmProject;
import com.nationrel.modules.project.entity.PmTg;
import com.nationrel.modules.project.jar.rb_common.common.constant.Constants;
import com.nationrel.modules.project.jar.rb_common.common.constant.MessageId;
import com.nationrel.modules.project.jar.rb_common.common.constant.ReliaFlowConstants;
import com.nationrel.modules.project.jar.rb_common.exception.ApplicationRuntimeException;
import com.nationrel.modules.project.jar.rb_common.exception.OptimisticLockException;
import com.nationrel.modules.project.jar.util.DateUtils;
import com.nationrel.modules.project.mapper.*;
import com.nationrel.modules.project.service.PmTaskService;
import com.nationrel.modules.project.service.PmTgService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @version V1.0
 * @ClassName PmTgServiceImpl
 * @Description TG服务接口
 * @Author andy
 * @CreateDate 2014-4-3
 */
@Service
public class PmTgServiceImpl implements PmTgService {
    @Autowired
    private PmProjectMapper pmProjectMapper;
    @Autowired
    private PmTgMapper pmTgMapper;
    @Autowired
    private PmTaskTreeMapper pmTaskTreeMapper;
    @Autowired
    private PmTgCheckMapper pmTgCheckMapper;
    @Autowired
    private PmTgDeliveryMapper pmTgDeliveryMapper;
    @Autowired
    private PmTgStaffMapper pmTgStaffMapper;
    @Autowired
    private PmTaskService pmTaskService;

    /**
     * @param tgId: TG标识
     * @return MessageId:如果返回W300103,则表示TG不存在;
     * 其他情况,返回null.
     * @Title: checkTgExist
     * @Description:检查TG是否存在
     */
    @Override
    public void checkTgExist(String tgId) throws Exception {
        // 获取Tg信息
        PmTg pmTg = pmTgMapper.selectByPrimaryKey(tgId);

        if (pmTg == null) {
            throw new OptimisticLockException(MessageId.W300103);
        }
    }

    /**
     * @param tgId void
     * @Title: deleteWholeTG
     * @Description: 删除整个TG
     */
    @Override
    public void deleteWholeTG(String tgId) {
        if (tgId == null || tgId.equals(Constants.BLANK)) {
            return;
        }

        // 调用PmTaskService删除TG下所有任务
        pmTaskService.deleteWholeTaskByTgId(tgId);

        // 删除TG下检查列表
        pmTgCheckMapper.deleteCheckItemByTgId(tgId);

        // 删除TG下成员列表
        pmTgStaffMapper.deleteStaffByTgId(tgId);

        // 删除TG下交付列表
        pmTgDeliveryMapper.deleteDeliveryByTgId(tgId);

        // 删除TG信息
        pmTgMapper.deleteByPrimaryKey(tgId);

    }

    /**
     * @param dtPlanStartTime
     * @param dtPlanEndTime
     * @param projectId
     * @param string          void
     * @Title: checkTimeValid
     * @Description: 检查TG时间的合法性
     */
    @Override
    public void checkTimeValid(Date dtPlanStartTime, Date dtPlanEndTime, String projectId, String tgId) throws Exception {

        // 1.当前TG的时间范围在项目时间范围之内
        PmProject projectEntity = pmProjectMapper.checkContainCurTimeOfTg(dtPlanStartTime, dtPlanEndTime, projectId);
        if (projectEntity == null) {
            throw new ApplicationRuntimeException(MessageId.E300116);
        }

        // 当存在创建状态以外的TG时，新建TG的开始时间必须晚于创建以外的TG的预计结束时间或实际完成时间
        List<PmTg> pmExTgs = pmTgMapper.getExCreateTg(projectId, tgId);
        if (pmExTgs.size() > 0) {
            PmTg entityTg = pmExTgs.get(pmExTgs.size() - 1);
            if (ReliaFlowConstants.LIFECYCLE_RELIA_TG_COMPLETE.equals(entityTg.getLifeState())) {
                if (DateUtils.compareTo(dtPlanStartTime, entityTg.getEndTime()) < 0) {
                    throw new ApplicationRuntimeException(MessageId.E300123, entityTg.getTgName());
                }
            } else {
                if (DateUtils.compareTo(dtPlanStartTime, entityTg.getPlanEndTime()) < 0) {
                    throw new ApplicationRuntimeException(MessageId.E300122, entityTg.getTgName());
                }
            }
        }

        // 2.当前TG的时间不能与其他TG时间重合
        List<PmTg> pmTgs = pmTgMapper.checkRepeatWithOtherTg(dtPlanStartTime, dtPlanEndTime, projectId, tgId);
        if (pmTgs.size() > 0) {
            throw new ApplicationRuntimeException(MessageId.E300117);
        }

        // 3.修改时-当前TG的时间必须包括所有子项的时间
        if (StringUtils.isNotEmpty(tgId)) {
            int errorTimeTasks = pmTaskTreeMapper.checkContainTimeOfTask(dtPlanStartTime, dtPlanEndTime, tgId);
            if (errorTimeTasks > 0) {
                throw new ApplicationRuntimeException(MessageId.E300118);
            }
        }
    }
}
