package com.nationrel.modules.project.service.impl;

import com.alibaba.fastjson.JSON;
import com.nationrel.modules.project.config.SysConfig;
import com.nationrel.modules.project.service.RedisService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * redis操作业务
 *
 * @author fmea
 */
@Slf4j
@Service
public class RedisServiceImpl implements RedisService {

    private final static String prefix = "%C2%AC%C3%AD%00%05";
    @Resource
    private RedisTemplate redisTemplate;
    @Autowired
    private SysConfig sysConfig;

    /**
     * @param val1
     * @param clazz
     * @return java.lang.Object
     * @description getObject
     * @author jingrx
     * @date 2021/7/13 10:19
     **/
    @Override
    public Object getObject(String key, Class clazz) {
        ValueOperations<String, String> ops = redisTemplate.opsForValue();
        String k = sysConfig.getSysName() + key;
        log.debug(String.format("getObject key=%s", k));
        String res = ops.get(k);
        Object ret = objectDeserialization(res);
        log.debug(String.format("getObject value=%s", ret != null ? JSON.toJSONString(ret) : null));
        return ret;
    }

    /**
     * @param val1
     * @param clazz
     * @description putObject
     * @author jingrx
     * @date 2021/7/13 10:19
     **/
    @Override
    public void putObject(String key, Object value, long timeout) {
        ValueOperations<String, String> ops = redisTemplate.opsForValue();
        String k = sysConfig.getSysName() + key;
        log.debug(String.format("putObject key=%s", k));
        log.debug(String.format("putObject value=%s", value != null ? JSON.toJSONString(value) : null));
        String v = objectSerialiable(value);
        ops.set(k, v, timeout, TimeUnit.SECONDS);
    }

    /**
     * @param key
     * @description delete
     * @author jingrx
     * @date 2021/7/13 10:19
     **/
    @Override
    public void delete(String key) {
        redisTemplate.delete(sysConfig.getSysName() + key);
    }

    /**
     * @param val1
     * @param val2
     * @return java.lang.Object
     * @description getObject
     * @author zk
     * @date 2021/3/17 10:19
     **/
    @Override
    public Object getObject(String val1, String val2) {
        return redisTemplate.opsForHash().get(sysConfig.getSysName() + val1, val2);
    }

    /**
     * @param val1
     * @param val2
     * @param value
     * @return void
     * @description setObject
     * @author zk
     * @date 2021/3/17 10:18
     **/
    @Override
    public void setObject(String val1, String val2, Object value) {
        redisTemplate.opsForHash().put(sysConfig.getSysName() + val1, val2, value);
    }

    /**
     * @param val1
     * @param val2
     * @param value
     * @return void
     * @description Redis-->hash中 插入Map
     * @author zk
     * @date 2021/2/20 14:48
     **/
    @Override
    public void setMap(String val1, String val2, Map<String, String> value) {
        redisTemplate.opsForHash().put(sysConfig.getSysName() + val1, val2, value);
    }

    /**
     * @param val1
     * @param val2
     * @return java.util.Map<java.lang.String, java.lang.String>
     * @description Redis-->hash中 获取Map
     * @author zk
     * @date 2021/2/20 14:50
     **/
    @Override
    public Map<String, String> getMap(String val1, String val2) {
        return (Map) redisTemplate.opsForHash().get(sysConfig.getSysName() + val1, val2);
    }

    /**
     * @param val1
     * @param val2
     * @return void
     * @description Redis-->hash中 删除Map
     * @author zk
     * @date 2021/2/4 17:38
     **/
    @Override
    public void delMap(String val1, String val2) {
        redisTemplate.opsForHash().delete(sysConfig.getSysName() + val1, val2);
    }

    /**
     * @param val1
     * @return void
     * @description Redis-->hash中 删除Map
     * @author zk
     * @date 2021/4/27 10:54
     **/
    @Override
    public void delMap(String val1) {
        redisTemplate.delete(sysConfig.getSysName() + val1);
    }

    @Override
    public Boolean hasMap(String val1) {
        return redisTemplate.opsForHash().getOperations().hasKey(sysConfig.getSysName() + val1);
    }

    @Override
    public Boolean setHashExpire(String val1, Long keepTime, TimeUnit timeUnit) {
        return redisTemplate.expire(sysConfig.getSysName() + val1, keepTime, timeUnit);
    }

    /**
     * 序列化
     *
     * @param obj
     * @return
     */
    public String objectSerialiable(Object obj) {
        String serStr = null;
        if (obj != null && obj instanceof String) {
            serStr = (String) obj;
        } else if (obj != null) {
            ByteArrayInputStream byteArrayInputStream = null;
            ObjectInputStream objectInputStream = null;
            try {
                ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
                ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
                objectOutputStream.writeObject(obj);
                serStr = byteArrayOutputStream.toString(StandardCharsets.ISO_8859_1);
                serStr = java.net.URLEncoder.encode(serStr, StandardCharsets.UTF_8);
            } catch (Exception e) {
                log.error(" objectSerialiable error ", e);
            } finally {
                if (objectInputStream != null) {
                    try {
                        objectInputStream.close();
                    } catch (IOException e) {
                        log.error(" objectSerialiable.objectInputStream.close() error ", e);
                    }
                }
                if (byteArrayInputStream != null) {
                    try {
                        byteArrayInputStream.close();
                    } catch (IOException e) {
                        log.error(" objectSerialiable.byteArrayInputStream.close() error ", e);
                    }
                }
            }
        }

        return serStr;
    }

    /**
     * 反序列化
     *
     * @param serStr
     * @return
     */
    public Object objectDeserialization(String serStr) {
        Object newObj = null;
        if (!StringUtils.isEmpty(serStr)) {
            if (!serStr.startsWith(prefix)) {
                newObj = serStr;
            } else {
                ByteArrayInputStream byteArrayInputStream = null;
                ObjectInputStream objectInputStream = null;
                try {
                    String redStr = java.net.URLDecoder.decode(serStr, StandardCharsets.UTF_8);
                    byteArrayInputStream = new ByteArrayInputStream(redStr.getBytes(StandardCharsets.ISO_8859_1));
                    objectInputStream = new ObjectInputStream(byteArrayInputStream);
                    newObj = objectInputStream.readObject();
                } catch (Exception e) {
                    log.error(" objectDeserialization error ", e);
                } finally {
                    if (objectInputStream != null) {
                        try {
                            objectInputStream.close();
                        } catch (IOException e) {
                            log.error(" objectDeserialization.objectInputStream.close() error ", e);
                        }
                    }
                    if (byteArrayInputStream != null) {
                        try {
                            byteArrayInputStream.close();
                        } catch (IOException e) {
                            log.error(" objectDeserialization.byteArrayInputStream.close() error ", e);
                        }
                    }
                }
            }
        }
        return newObj;
    }

    @Override
    public RedisTemplate getRedisTemplate() {
        return redisTemplate;
    }
}
